import { createConvertToContextFrame } from './factories/convert-to-context-frame';
import { createPerformance } from './factories/performance';
import { createScheduleAudioBufferSourceNode } from './factories/schedule-audio-buffer-source-node';
import { createSubscribeToTimingObject } from './factories/subscribe-to-timing-object';
import { worklet } from './worklet/worklet';
/*
 * @todo Explicitly referencing the barrel file seems to be necessary when enabling the
 * isolatedModules compiler option.
 */
export * from './interfaces/index';
export * from './types/index';
const blob = new Blob([worklet], { type: 'application/javascript; charset=utf-8' });
export const addTimedAudioBufferSourceNodeAudioWorkletModule = async (addAudioWorkletModule) => {
    const url = URL.createObjectURL(blob);
    try {
        await addAudioWorkletModule(url);
    }
    finally {
        URL.revokeObjectURL(url);
    }
};
const convertToContextFrame = createConvertToContextFrame(createPerformance());
const subscribeToTimingObject = createSubscribeToTimingObject(createScheduleAudioBufferSourceNode(convertToContextFrame));
export function createTimedAudioBufferSourceNodeAudioWorkletNode(audioWorkletNodeConstructor, context, createAudioBufferSourceNode, options = {}) {
    var _a, _b;
    const { buffer = null } = options;
    if (buffer instanceof AudioBuffer && buffer.sampleRate !== context.sampleRate) {
        throw new TypeError('The AudioBuffer must have the same sampleRate as the AudioContext.');
    }
    let { timingObject = null } = options;
    const { position = 0, timestamp = 0, velocity = 0 } = (_a = timingObject === null || timingObject === void 0 ? void 0 : timingObject.query()) !== null && _a !== void 0 ? _a : {};
    const audioWorkletNode = new audioWorkletNodeConstructor(context, 'timed-audio-buffer-source-node-audio-worklet-processor', {
        numberOfInputs: 1,
        numberOfOutputs: 1,
        outputChannelCount: [(_b = buffer === null || buffer === void 0 ? void 0 : buffer.numberOfChannels) !== null && _b !== void 0 ? _b : 1],
        processorOptions: {
            buffer: buffer instanceof AudioBuffer
                ? Array.from({ length: buffer.numberOfChannels }, (_, channel) => buffer.getChannelData(channel))
                : null,
            position,
            timestamp: convertToContextFrame(context, timestamp),
            velocity
        }
    });
    let removeListener = null;
    Object.defineProperties(audioWorkletNode, {
        port: {
            get() {
                throw new Error("The port of a TimedAudioBufferSourceNodeAudioWorkletNode can't be accessed.");
            }
        },
        timingObject: {
            get: () => timingObject,
            set: (value) => {
                removeListener === null || removeListener === void 0 ? void 0 : removeListener();
                removeListener = null;
                if (value === null) {
                    timingObject = value;
                }
                else {
                    throw new TypeError('A TimingObject can only be set in the constructor.');
                }
            }
        }
    });
    if (timingObject !== null) {
        removeListener = subscribeToTimingObject(audioWorkletNode, context, createAudioBufferSourceNode, timingObject);
    }
    return audioWorkletNode;
}
//# sourceMappingURL=module.js.map